@@ -1,7 +1,7 @@
 from .support import TickLiveData, MinLiveData, TouchlineData
 from .support import TDLiveDataError, TDHistoricDataError
 from .TD_live import LiveClient
-from .TD_hist import HistoricalWebsocket
+from .TD_hist import HistoricalREST
 from threading import Thread
 from typing import Union
 from collections import defaultdict
@@ -13,15 +13,28 @@ import json
 
 from colorama import Style, Fore
 import logging
+from typing import List, Dict
 
 
 class TD:
-    def __init__(self, login_id, password, broker_token=None, url='push.truedata.in', live_port=8082, historical_port=8092, logger_propagate=False, tz='Asia/Kolkata', logger_level=20, logger=None):
+    def __init__(self,
+                 login_id,
+                 password,
+                 broker_token=None,
+                 url='push.truedata.in',  # This arg is used for LIVE URL only
+                 live_port=8082,
+                 historical_port=8092,
+                 logger_propagate=False,
+                 # tz='Asia/Kolkata',
+                 logger_level=20,
+                 logger=None,
+                 hist_url='https://history.truedata.in'):
         self.live_websocket = None
-        self.historical_websocket = None
+        self.historical_datasource = None
         self.login_id = login_id
         self.password = password
-        self.url = url
+        self.live_url = url
+        self.hist_url = hist_url
         self.live_port = live_port
         self.historical_port = historical_port
         if logger is None:
@@ -53,11 +66,11 @@ class TD:
         if self.broker_token is not None:
             broker_append = f'&brokertoken={self.broker_token}'
         if self.connect_live:
-            self.live_websocket = LiveClient(self, f"wss://{self.url}:{self.live_port}?user={self.login_id}&password={self.password}{broker_append}")
+            self.live_websocket = LiveClient(self, f"wss://{self.live_url}:{self.live_port}?user={self.login_id}&password={self.password}{broker_append}")
             t = Thread(target=self.connect_thread, args=(), daemon=True)
             t.start()
         if self.connect_historical:
-            self.historical_websocket = HistoricalWebsocket(self.login_id, self.password, self.url, self.historical_port, self.broker_token, self.logger)
+            self.historical_datasource = HistoricalREST(self.login_id, self.password, self.hist_url, self.historical_port, self.broker_token, self.logger)
         while self.connect_live and self.live_websocket.subscription_type == '':
             time.sleep(1)
 
@@ -72,9 +85,8 @@ class TD:
             # self.logger.info(f"{Fore.GREEN}Disconnected LIVE TrueData...{Style.RESET_ALL}")
             print(f"{Style.NORMAL}{Fore.BLUE}Disconnected from Real Time Data WebSocket Connection !{Style.RESET_ALL}")
         if self.connect_historical:
-            self.historical_websocket.hist_socket.close()
             # self.logger.info(f"{Fore.GREEN}Disconnected HISTORICAL TrueData...{Style.RESET_ALL}")
-            print(f"{Style.NORMAL}{Fore.BLUE}Disconnected from Historical Data WebSocket Connection !{Style.RESET_ALL}")
+            print(f"{Style.NORMAL}{Fore.BLUE}Disconnected from Historical Data REST Connection !{Style.RESET_ALL}")
 
     @staticmethod
     def truedata_duration_map(regular_format, end_date):
@@ -96,29 +108,42 @@ class TD:
                           end_time=None,
                           duration=None,
                           start_time=None,
-                          bar_size="1 min"):
+                          bar_size="1 min",
+                          options=None,
+                          bidask=False):
         if start_time is not None and duration is None:
             return self.get_historical_data_from_start_time(contract=contract,
                                                             ticker_id=ticker_id,
                                                             end_time=end_time,
                                                             start_time=start_time,
-                                                            bar_size=bar_size)
+                                                            bar_size=bar_size,
+                                                            options=options,
+                                                            bidask=bidask)
         else:
             return self.get_historical_data_from_duration(contract=contract,
                                                           ticker_id=ticker_id,
                                                           end_time=end_time,
                                                           duration=duration,
-                                                          bar_size=bar_size)
+                                                          bar_size=bar_size,
+                                                          options=options,
+                                                          bidask=bidask)
 
     def get_n_historical_bars(self, contract,
                               ticker_id=None,
                               end_time: datetime = None,
                               no_of_bars: int = 1,
-                              bar_size="1 min"):
+                              bar_size="1 min",
+                              options=None,
+                              bidask=False):
         if end_time is None:
             end_time = datetime.today()
-        end_time = end_time.strftime('%Y-%m-%dT%H:%M:%S')
-        hist_data = self.historical_websocket.get_n_historic_bars(contract, end_time, no_of_bars, bar_size)
+        end_time = end_time.strftime('%y%m%dT%H:%M:%S')    # This is the request format
+        hist_data = self.historical_datasource.get_n_historic_bars(contract,
+                                                                   end_time,
+                                                                   no_of_bars,
+                                                                   bar_size,
+                                                                   options=options,
+                                                                   bidask=bidask)
         if ticker_id is None:
             ticker_id = self.default_historical_data_id
             self.default_historical_data_id = self.default_historical_data_id + 1
@@ -129,16 +154,18 @@ class TD:
                                           ticker_id=None,
                                           end_time: datetime = None,
                                           duration=None,
-                                          bar_size="1 min"):
+                                          bar_size="1 min",
+                                          options=None,
+                                          bidask=False):
         if duration is None:
             duration = "1 D"
         if end_time is None:
             end_time = datetime.today()
         start_time = self.truedata_duration_map(duration, end_time)
-        end_time = end_time.strftime('%Y-%m-%d') + 'T23:59:59'
-        start_time = start_time.strftime('%Y-%m-%d') + 'T00:00:00'
+        end_time = end_time.strftime('%y%m%d') + 'T23:59:59'    # This is the request format
+        start_time = start_time.strftime('%y%m%d') + 'T00:00:00'    # This is the request format
 
-        hist_data = self.historical_websocket.get_historic_data(contract, end_time, start_time, bar_size)
+        hist_data = self.historical_datasource.get_historic_data(contract, end_time, start_time, bar_size, options=options, bidask=bidask)
         if ticker_id is None:
             ticker_id = self.default_historical_data_id
             self.default_historical_data_id = self.default_historical_data_id + 1
@@ -149,21 +176,29 @@ class TD:
                                             ticker_id=None,
                                             end_time: datetime = None,
                                             start_time: datetime = None,
-                                            bar_size="1 min"):
+                                            bar_size="1 min",
+                                            options=None,
+                                            bidask=False):
         if end_time is None:
             end_time = datetime.today().replace(hour=23, minute=59, second=59)
         if start_time is None:
             start_time = datetime.today().replace(hour=0, minute=0, second=0)
 
-        end_time = end_time.strftime('%Y-%m-%dT%H:%M:%S')
-        start_time = start_time.strftime('%Y-%m-%dT%H:%M:%S')
-        hist_data = self.historical_websocket.get_historic_data(contract, end_time, start_time, bar_size)
+        end_time = end_time.strftime('%y%m%dT%H:%M:%S')    # This is the request format
+        start_time = start_time.strftime('%y%m%dT%H:%M:%S')    # This is the request format
+        hist_data = self.historical_datasource.get_historic_data(contract, end_time, start_time, bar_size, options=options, bidask=bidask)
         if ticker_id is None:
             ticker_id = self.default_historical_data_id
             self.default_historical_data_id = self.default_historical_data_id + 1
         self.hist_data[ticker_id] = hist_data
         return hist_data
 
+    def get_bhavcopy(self, segment: str, date: datetime = None, return_completed: bool = True) -> List[Dict]:
+        if date is None:
+            date = datetime.now().replace(hour=0, minute=0, second=0)
+        # return_completed = True  # Uncomment and remove arg to disallow user choice
+        return self.historical_datasource.bhavcopy(segment, date, return_completed)
+
     @staticmethod
     def get_req_id_list(req_id: Union[int, list], len_contracts: int) -> list:
         if type(req_id) == list:
