@@ -81,7 +81,8 @@ class LiveClient(WebSocketApp):
             is_td_connected = self.check_connection()
         return is_td_connected
 
-    def on_msg_func(self, message):
+    def on_msg_func(self, *args):
+        message = args[-1]
         msg = json.loads(message)
         if 'message' in msg.keys():
             self.handle_message_data(msg)
@@ -241,12 +242,13 @@ class LiveClient(WebSocketApp):
         except Exception as e:
             self.logger.error(f'{Style.BRIGHT}{Fore.RED}Bar feed encountered - {e}{Style.RESET_ALL}')
 
-    def ws_error(self, error):
+    def ws_error(self, *args):
+        error = args[-1]
         if any(isinstance(error, conn_error) for conn_error in [ConnectionResetError, TimeoutError]):
             self.logger.error(f"Raising WS error = {error}")
             self.last_ping_tm = self.last_pong_tm = 0
 
-    def ws_open(self):
+    def ws_open(self, *args):
         self.last_ping_tm = time.time()
         self.sock.ping()
         self.sock.settimeout(15)
@@ -254,7 +256,7 @@ class LiveClient(WebSocketApp):
             print('Some data needs resuming here')
             # print(self.parent_app.symbol_mkt_id_map)
 
-    def ws_close(self):
+    def ws_close(self, *args):
         # self.logger.error('Live WebSocket Closed')
         self.sock.close()
         self.sock = None
